import { BrowserContext, ElectronApplication, Page, _electron as electron } from 'playwright';
import { test, expect } from '@playwright/test';
import * as PATH from 'path';
import {GameModel} from "../../../src/app/game-model/GameModel";
import {Gamesystem} from "../../src/app/game-model/gamesystems/Gamesystem";
import {ScriptAccount} from "../../../src/app/game-model/scriptAccounts/ScriptAccount";
import {ModelComponentType} from "../../../src/app/game-model/ModelComponentType";
import {SimpleGamesystem} from "../../../src/app/game-model/gamesystems/SimpleGamesystem";
import exp = require("node:constants");
import {end} from "electron-debug";
import {GamesystemTrainer} from "./GamesystemTrainer";
import {ProductGamesystem} from "../../../src/app/game-model/gamesystems/ProductGamesystem";
import {ProductStateTrainer} from "./ProductStateTrainer";
import {SimpleState} from "../../../../src/app/game-model/gamesystems/SimpleState";
test.describe('Test Create ProductStates', () => {

  test("Adding already existent ProductState", async () => {
    const gamesystem = ProductStateTrainer.givenFullProductGamesystemWithTwoStates();
    const result = gamesystem.createState(gamesystem.states[0].innerStates);
    expect(result).toBeUndefined();
    expect(gamesystem.states.length).toEqual(4);
  })

  test("Test empty inputs for ProductState Creation", async ()=> {
    const gamesystem = ProductStateTrainer.givenFullProductGamesystemWithTwoStates();
    expect(gamesystem.createState([])).toBeUndefined();
    expect(gamesystem.states.length).toEqual(4);
  })

  test("Test invalid inputs for ProductState Creation", async () => {
    const gamesystem = ProductStateTrainer.givenFullProductGamesystemWithTwoStates();
    expect(gamesystem.createState([ProductStateTrainer.givenUnusedInnerStates(gamesystem)[0]])).toBeUndefined();
    expect(gamesystem.states.length).toEqual(4);
  })

  test("Test ProductState Creation with foreign State", async () => {
    const gamesystem = ProductStateTrainer.givenFullProductGamesystemWithTwoStates();
    const simpleState = new SimpleState(ProductStateTrainer.INNERSTATE_LETTER_3);
    expect(gamesystem.createState([simpleState])).toBeUndefined();
    expect(gamesystem.states.length).toEqual(4);
  })

  test("Test valid ProductStateCreation", async () => {
    const gamesystem = ProductStateTrainer.givenFullProductGamesystemWithTwoStates();
    expect(gamesystem.createState(ProductStateTrainer.givenUnusedInnerStates(gamesystem))).toBeDefined();
    expect(gamesystem.states.length).toEqual(5);
  })
});
