import {Component, EventEmitter, Input, Output} from '@angular/core';
import {GameModel} from "../game-model/GameModel";
import {ModelComponent} from "../game-model/ModelComponent";
import {ModelComponentType} from "../game-model/ModelComponentType";
import {ScriptAccount} from "../game-model/scriptAccounts/ScriptAccount";
import {Gamesystem} from "../game-model/gamesystems/Gamesystem";
import {State} from "../game-model/gamesystems/State";
import {Transition} from "../game-model/gamesystems/Transition";

@Component({
  selector: 'app-editor',
  templateUrl: './editor.component.html',
  styleUrl: './editor.component.scss'
})
export class EditorComponent {
  gameModelComponents: ModelComponent[] = [];
  @Output("onModelNameUpdate") onModelNameUpdateEmitter = new EventEmitter<boolean>();
  activeTab: number = this.gameModelComponents.length;

  openGameModelComponent(gameModelComponent: ModelComponent) {
    if(!this.gameModelComponents.includes(gameModelComponent)) {
      this.gameModelComponents.push(gameModelComponent);
      this.activeTab = this.gameModelComponents.length;
    } else {
      this.activeTab = this.gameModelComponents.findIndex(component => component.componentName === gameModelComponent.componentName);
    }
  }

  closeGameModelComponent(gameModelComponent: ModelComponent) {
    this.gameModelComponents = this.gameModelComponents.filter(modelComponent => modelComponent !== gameModelComponent);
  }

  convertModelComponentToScriptAccount(modelComponent: ModelComponent) {
    if(modelComponent instanceof ScriptAccount) {
      return modelComponent as ScriptAccount;
    }
  }

  convertModelComponentToGamesystem(modelComponent: ModelComponent) {
    if(modelComponent instanceof Gamesystem) {
      return modelComponent as Gamesystem<State<any>, Transition<any>>;
    }
  }

  protected readonly ModelComponentType = ModelComponentType;

  onModelNameUpdate() {
    this.onModelNameUpdateEmitter.emit(true);
  }
}
